package de.cliffbreak.varo.managers;

import org.bukkit.Bukkit;
import org.bukkit.World;
import org.bukkit.World.Environment;
import org.bukkit.entity.Creeper;
import org.bukkit.entity.Drowned;
import org.bukkit.entity.Enderman;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.bukkit.entity.Skeleton;
import org.bukkit.entity.Spider;
import org.bukkit.entity.Zombie;

import de.cliffbreak.varo.Varo;
import de.cliffbreak.varo.enums.GameState;

public class GameManager {

    private final Varo plugin;
    private GameState gameState;

    public GameManager(final Varo plugin) {
        this.plugin = plugin;
        this.gameState = GameState.NOTIME;
    }

    /**
     * @return current GameState
     */
    public GameState getGameState() {
        return this.gameState;
    }

    /**
     * Set this to allow all players to login and show them a loading screen
     */
    public void setLoginPhase() {
        if (this.gameState.equals(GameState.LOGIN))
            return;
        this.gameState = GameState.LOGIN;
        this.plugin.getLogger().info("[GameManager] Set GameState to " + this.gameState);
    }

    /**
     * Set to prestart phase (teleport all players to their start locations and show 1 minute countdown)
     */
    public void setPreStart() {
        if (this.gameState.equals(GameState.PRESTART))
            return;
        this.gameState = GameState.PRESTART;
        this.plugin.getLogger().info("[GameManager] Set GameState to " + this.gameState);
        for (World world : Bukkit.getWorlds()) {
            world.setPVP(false);
            world.setTime(0);
            world.setStorm(false);

            if (world.getEnvironment().equals(Environment.NORMAL)) {
                for (Entity entity : world.getEntitiesByClasses(Spider.class, Zombie.class, Skeleton.class,
                        Creeper.class, Drowned.class, Enderman.class)) {
                    entity.remove();
                    // TODO: test
                }
            }
        }
        for (Player player : Bukkit.getOnlinePlayers()) {
            player.setHealth(20.0d);
            player.setFoodLevel(20);
            player.setSaturation(5.0f);
        }
        // Telport handled from PlayerUtils.java
    }

    /**
     * Set this on day one to disable PVP
     */
    public void setProtectionPhase() {
        if (this.gameState.equals(GameState.PROTECTION))
            return;
        this.gameState = GameState.PROTECTION;
        this.plugin.getLogger().info("[GameManager] Set GameState to " + this.gameState);
        for (World world : Bukkit.getWorlds()) {
            world.setPVP(false);
        }
    }

    /**
     * Set this to teleport all players to the normal world and start the game as usual (for days > 1)
     */
    public void setIngame() {
        if (this.gameState.equals(GameState.INGAME))
            return;
        this.gameState = GameState.INGAME;
        this.plugin.getLogger().info("[GameManager] Set GameState to " + this.gameState);
        for (World world : Bukkit.getWorlds()) {
            world.setPVP(true);
        }
    }

    /**
     * Set this if the PlayTime is over, this will also kick the Players
     */
    public void setGameTimeOver() {
        if (this.gameState.equals(GameState.NOTIME))
            return;
        this.gameState = GameState.NOTIME;
        this.plugin.getLogger().info("[GameManager] Set GameState to " + this.gameState);
        for (Player player : Bukkit.getOnlinePlayers()) {
            System.out.println(player.getName() + " kick..");
        }
    }
}